
from src.motors.gimbal_motor_driver import GimbalMotorDriver
from src.motors.claw_motor_driver import ClawMotorDriver
from src.motors.stepper_motor_driver import StepperMotorDriver
from src.managers.sensor_manager import SensorManager
from src.gaits.drive_gait import DriveGait
from src.gaits.turn_gait import TurnGait

import RPi.GPIO as GPIO

import math
import time

class SetupMotorManager():
    """
    Class for driving motors for the robot gait based off given drive commands
    """
    CW = "CW"
    CCW = "CCW"
    
    
    CLAW_SPEED = 0
    SCREW_SPEED = 0
    
    def __init__(self, sensorManager):
        """
        Initialize the motorManager.
        
        sensorManager - manager for looking at sensor data
        """
        self.currentPitch = 0
        self.currentYaw = 0
        self.steps = 0
        self.sensorManager = sensorManager
        self.gpio_expander = sensorManager.gpio_expander
        self.clawMotorTop = ClawMotorDriver(12, 26, 16, self.gpio_expander)
        self.clawMotorBottom = ClawMotorDriver(13, 21, 20, self.gpio_expander)
        self.gimbleMotorYaw1 = GimbalMotorDriver(8, 17, 27, self.gpio_expander)
        self.gimbleMotorYaw2 = GimbalMotorDriver(9, 12, 5, self.gpio_expander)
        self.gimbleMotorPitch1 = GimbalMotorDriver(15, 6, 24, self.gpio_expander)
        self.gimbleMotorPitch2 = GimbalMotorDriver(11, 13, 19, self.gpio_expander)
        self.screwMotor = StepperMotorDriver(25, 15, 18)
        self.upGait = DriveGait("UP", sensorManager, self)
        self.downGait = DriveGait("DOWN", sensorManager, self)
        self.leftGait = TurnGait("LEFT", sensorManager, self)
        self.rightGait = TurnGait("RIGHT", sensorManager, self)

    def driveClaws(self, robotHalf, direction):
        """
        Attach the claws of the given half
        
        distance - specified half (TOP or BOTTOM)
        """
        if(robotHalf == "TOP"):
            clawMotor = self.clawMotorTop
        else:
            clawMotor = self.clawMotorBottom
        
        
        if(direction):
            clawMotor.reverse()
        else:
            clawMotor.write()
            
        
    def stopClaws(self, robotHalf):
        """
        Attach the claws of the given half
        
        distance - specified half (TOP or BOTTOM)
        """
        if(robotHalf == "TOP"):
            clawMotor = self.clawMotorTop
        else:
            clawMotor = self.clawMotorBottom
        
        clawMotor.stop()
        
    def driveGimbalPitch(self, gimbalMotorSelect, direction, duty=100):
        """
        Drive specified gimbal motor
        
        specified motor TOP or BOTTOM
        """

        if(gimbalMotorSelect == "TOP"):
            gimbalMotor = self.gimbleMotorPitch1
        else:
            gimbalMotor = self.gimbleMotorPitch2
        if(direction):
            gimbalMotor.write("CW", duty)
        else:
            gimbalMotor.write("CCW", duty)
            
    def driveGimbalYaw(self, gimbalMotorSelect, direction, duty=100):
        """
        Drive specified gimbal motor
        
        specified motor  LEFT or RIGHT
        """
        if(gimbalMotorSelect == "LEFT"):
            gimbalMotor = self.gimbleMotorYaw1
        else:
            gimbalMotor = self.gimbleMotorYaw2

        if(direction):
            gimbalMotor.write("CW", duty)
        else:
            gimbalMotor.write("CCW", duty)
    
    def setGimbalMotor(self, motor, pot, target, potMin, potMax, motorDir):
        """
        Drive the specified motor to the specified pot value 
        needs the min and max pot values for that motor and the direction
        """
        potDirection = 0
        validTarget = 0
        if (potMin > potMax):
            potDirection = 1
        
        if (potDirection):
            if (potMin >= target and target >= potMax):
                validTarget = 1
        else:
            if (potMin <= target and target <= potMax):
                validTarget = 1
        if validTarget:
            current = pot.readValue()
            diff = 0
            attempt = 0
            while (current != target and (diff != 0 or attempt == 0)):
                attempt = 1
                # min value is less
                if current < target and not potDirection:
                    # Normal direction, drive towards max
                    if(motor == "TOP" or motor == "BOT"):
                        self.driveGimbalPitch(motor, motorDir, 100)
                    else:
                        self.driveGimbalYaw(motor, motorDir, 100)
                if current > target and not potDirection:
                    # opposite direction, drive towards min
                    if(motor == "TOP" or motor == "BOT"):
                        self.driveGimbalPitch(motor, not motorDir, 100)
                    else:
                        self.driveGimbalYaw(motor, not motorDir, 100)
                # min value is greater
                if current > target and potDirection:
                    # Normal direction
                    if(motor == "TOP" or motor == "BOT"):
                        self.driveGimbalPitch(motor, motorDir, 100)
                    else:
                        self.driveGimbalYaw(motor, motorDir, 100)
                if current < target and potDirection:
                    # opposite direction
                    if(motor == "TOP" or motor == "BOT"):
                        self.driveGimbalPitch(motor, not motorDir, 100)
                    else:
                        self.driveGimbalYaw(motor, not motorDir, 100)
                last = current
                current = pot.readValue()
                diff = last - current
            self.stop()

    def stopGimbal(self):
        self.gimbleMotorPitch1.write("", 0)
        self.gimbleMotorPitch2.write("", 0)
        self.gimbleMotorYaw1.write("", 0)
        self.gimbleMotorYaw2.write("", 0)
    
    def driveLeadScrew(self, direction, steps=5):
        """
        drive lead screw in specified direction
        
        direction - direction to drive in (RETRACT or EXTEND)
        steps - number of steps to drive in
        """
        if(direction == "RETRACT"):
            self.steps -= steps
        else:
            self.steps += steps
            
        self.screwMotor.write(direction, steps)
        self.sensorManager.setSteps(self.steps)
    
    
    def stop(self):
        """
        Stop the robot.
        """
        self.gimbleMotorPitch1.write("", 0)
        self.gimbleMotorPitch2.write("", 0)
        self.gimbleMotorYaw1.write("", 0)
        self.gimbleMotorYaw2.write("", 0)
        self.clawMotorTop.stop()
        self.clawMotorBottom.stop()
        self.screwMotor.stop()